@@ -1,23 +1,21 @@
-### dist.topo.R (2007-03-09)
-###
-###      Topological Distances, Tree Bipartitions,
-###   Consensus Trees, and Bootstrapping Phylogenies
-###
-### Copyright 2005-2007 Emmanuel Paradis
-###
-### This file is part of the R-package `ape'.
-### See the file ../COPYING for licensing issues.
+## dist.topo.R (2007-07-04)
+
+##      Topological Distances, Tree Bipartitions,
+##   Consensus Trees, and Bootstrapping Phylogenies
+
+## Copyright 2005-2007 Emmanuel Paradis
+
+## This file is part of the R-package `ape'.
+## See the file ../COPYING for licensing issues.
 
 dist.topo <- function(x, y, method = "PH85")
 {
     if (method == "BHV01" && (is.null(x$edge.length) || is.null(y$edge.length)))
       stop("trees must have branch lengths for Billera et al.'s distance.")
     n <- length(x$tip.label)
-    bp1 <- .Call("bipartition", x$edge[, 1], x$edge[, 2],
-                  n, x$Nnode, PACKAGE = "ape")
+    bp1 <- .Call("bipartition", x$edge, n, x$Nnode, PACKAGE = "ape")
     bp1 <- lapply(bp1, function(xx) sort(x$tip.label[xx]))
-    bp2 <- .Call("bipartition", y$edge[, 1], y$edge[, 2],
-                 n, y$Nnode, PACKAGE = "ape")
+    bp2 <- .Call("bipartition", y$edge, n, y$Nnode, PACKAGE = "ape")
     bp2 <- lapply(bp2, function(xx) sort(y$tip.label[xx]))
     q1 <- length(bp1)
     q2 <- length(bp2)
@@ -56,69 +54,103 @@ dist.topo <- function(x, y, method = "PH85")
     dT
 }
 
-prop.part <- function(...)
+prop.part <- function(..., check.labels = FALSE)
 {
     obj <- list(...)
     if (length(obj) == 1 && class(obj[[1]]) != "phylo")
       obj <- unlist(obj, recursive = FALSE)
     ntree <- length(obj)
-    bp <- .Call("bipartition", obj[[1]]$edge[, 1], obj[[1]]$edge[, 2],
-                length(obj[[1]]$tip.label), obj[[1]]$Nnode,
-                PACKAGE = "ape")
-    clades <- lapply(bp, function(xx) sort(obj[[1]]$tip.label[xx]))
-    no <- rep(1, length(clades))
+    if (!check.labels) {
+        for (i in 1:ntree) storage.mode(obj[[i]]$Nnode) <- "integer"
+        clades <- .Call("prop_part", obj, ntree, TRUE, PACKAGE = "ape")
+        attr(clades, "number") <- attr(clades, "number")[1:length(clades)]
+        attr(clades, "labels") <- obj[[1]]$tip.label
+    } else {
+        bp <- .Call("bipartition", obj[[1]]$edge, length(obj[[1]]$tip.label),
+                    obj[[1]]$Nnode, PACKAGE = "ape")
+        clades <- lapply(bp, function(xx) sort(obj[[1]]$tip.label[xx]))
+        no <- rep(1, length(clades))
 
-    if (ntree > 1) {
-        for (k in 2:ntree) {
-            bp <- .Call("bipartition", obj[[k]]$edge[, 1], obj[[k]]$edge[, 2],
-                        length(obj[[k]]$tip.label), obj[[k]]$Nnode,
-                        PACKAGE = "ape")
-            bp <- lapply(bp, function(xx) sort(obj[[k]]$tip.label[xx]))
-            for (i in 1:length(bp)) {
-                done <- FALSE
-                for (j in 1:length(clades)) {
-                    if (identical(all.equal(bp[[i]], clades[[j]]), TRUE)) {
-                        no[j] <- no[j] + 1
-                        done <- TRUE
-                        break
+        if (ntree > 1) {
+            for (k in 2:ntree) {
+                bp <- .Call("bipartition", obj[[k]]$edge,
+                            length(obj[[k]]$tip.label), obj[[k]]$Nnode,
+                            PACKAGE = "ape")
+                bp <- lapply(bp, function(xx) sort(obj[[k]]$tip.label[xx]))
+                for (i in 1:length(bp)) {
+                    done <- FALSE
+                    for (j in 1:length(clades)) {
+                        if (identical(all.equal(bp[[i]], clades[[j]]), TRUE)) {
+                            no[j] <- no[j] + 1
+                            done <- TRUE
+                            break
+                        }
+                    }
+                    if (!done) {
+                        clades <- c(clades, bp[i])
+                        no <- c(no, 1)
                     }
-                }
-                if (!done) {
-                    clades <- c(clades, bp[i])
-                    no <- c(no, 1)
                 }
             }
         }
+        attr(clades, "number") <- no
     }
-    attr(clades, "number") <- no
     class(clades) <- "prop.part"
     clades
 }
 
 print.prop.part <- function(x, ...)
 {
-    for (i in 1:length(x)) {
-        cat("==>", attr(x, "number")[i], "time(s):")
-        print(x[[i]], quote = FALSE)
+    if (is.null(attr(x, "labels"))) {
+        for (i in 1:length(x)) {
+            cat("==>", attr(x, "number")[i], "time(s):")
+            print(x[[i]], quote = FALSE)
+        }
+    } else {
+        for (i in 1:length(attr(x, "labels")))
+          cat(i, ": ", attr(x, "labels")[i], "\n", sep = "")
+        cat("\n")
+        for (i in 1:length(x)) {
+            cat("==>", attr(x, "number")[i], "time(s):")
+            print(x[[i]], quote = FALSE)
+        }
     }
 }
 
 summary.prop.part <- function(object, ...) attr(object, "number")
 
+plot.prop.part <- function(x, barcol = "blue", leftmar = 4, ...)
+{
+    if (is.null(attr(x, "labels")))
+      stop("cannot plot this partition object; see ?prop.part for details.")
+    L <- length(x)
+    n <- length(attr(x, "labels"))
+    layout(matrix(1:2, 2, 1), heights = c(1, 3))
+    par(mar = c(0.1, leftmar, 0.1, 0.1))
+    plot(1:L, attr(x, "number"), type = "h", col = barcol, xlim = c(1, L),
+         xlab = "", ylab = "Number", xaxt = "n", bty = "n")
+    plot(0, type = "n", xlim = c(1, L), ylim = c(1, n),
+         xlab = "", ylab = "", xaxt = "n", yaxt = "n")
+    for (i in 1:L) points(rep(i, length(x[[i]])), x[[i]], ...)
+    mtext(attr(x, "labels"), side = 2, at = 1:n, las = 1)
+}
+
 prop.clades <- function(phy, ..., part = NULL)
 {
     if (is.null(part)) {
         obj <- list(...)
         if (length(obj) == 1 && class(obj[[1]]) != "phylo")
           obj <- unlist(obj, recursive = FALSE)
-        part <- prop.part(obj)
+        part <- prop.part(obj, check.labels = TRUE)
     }
-    bp <- .Call("bipartition", phy$edge[, 1], phy$edge[, 2],
-                length(phy$tip.label), phy$Nnode,
-                PACKAGE = "ape")
+    bp <- .Call("bipartition", phy$edge, length(phy$tip.label),
+                phy$Nnode, PACKAGE = "ape")
+    if (!is.null(attr(part, "labels")))
+      for (i in 1:length(part))
+        part[[i]] <- sort(attr(part, "labels")[part[[i]]])
     bp <- lapply(bp, function(xx) sort(phy$tip.label[xx]))
-    n <- numeric(length(bp))
-    for (i in 1:length(bp)) {
+    n <- numeric(phy$Nnode)
+    for (i in 1:phy$Nnode) {
         for (j in 1:length(part)) {
             if (identical(all.equal(bp[[i]], part[[j]]), TRUE)) {
                 n[i] <- attr(part, "number")[j]
@@ -140,8 +172,7 @@ boot.phylo <- function(phy, x, FUN, B = 100, block = 1)
         x <- matrix(x, n, nL/n, byrow = TRUE)
         rownames(x) <- nm
     }
-    boot.tree <- list()
-    length(boot.tree) <- B
+    boot.tree <- vector("list", B)
     for (i in 1:B) {
         if (block > 1) {
             y <- seq(block, ncol(x), block)
@@ -153,7 +184,10 @@ boot.phylo <- function(phy, x, FUN, B = 100, block = 1)
         } else boot.samp <- sample(ncol(x), replace = TRUE)
         boot.tree[[i]] <- FUN(x[, boot.samp])
     }
-    prop.clades(phy, boot.tree)
+    for (i in 1:B) storage.mode(boot.tree[[i]]$Nnode) <- "integer"
+    storage.mode(phy$Nnode) <- "integer"
+    attr(.Call("prop_part", c(list(phy), boot.tree), B + 1, FALSE,
+               PACKAGE = "ape"), "number") - 1
 }
 
 consensus <- function(..., p = 1)
@@ -163,7 +197,7 @@ consensus <- function(..., p = 1)
       obj <- unlist(obj, recursive = FALSE)
     ntree <- length(obj)
     ## Get all observed partitions and their frequencies:
-    pp <- prop.part(obj)
+    pp <- prop.part(obj, check.labels = TRUE)
     ## Drop the partitions whose frequency is less than 'p':
     pp <- pp[attr(pp, "number") >= p * ntree]
     ## Get the order of the remaining partitions by decreasing size:
